#include <iostream.h>
#include "colorOpt.h"
#include "colorOpt.moc"

#include <qlabel.h>
#include <qframe.h>
#include <qlayout.h>
#include "kcolorbtn.h"
#include "ircDefine.h"
#include "ircApp.h"

ColorOpt::ColorOpt( QWidget *parent, const char *name ) : QWidget( parent, name )
{
  QHBoxLayout* horLayout  = new QHBoxLayout(this, 10);
  QVBoxLayout* lVerLayout = new QVBoxLayout(0);
  horLayout->addLayout(lVerLayout);

  QGroupBox* gb          = new QGroupBox("Color Options", this);
  lVerLayout->addWidget(gb);

  readOptions();
  
  KColorButton *colorBtn;
  QLabel *label;
  QBoxLayout* vlayout;
  QBoxLayout* hlayout;
  vlayout = new QVBoxLayout(gb, 10);
  vlayout->addSpacing(gb->fontMetrics().height());

  hlayout = new QHBoxLayout(5);
  vlayout->addLayout(hlayout);
  label = new QLabel( klocale->translate("Background Color:"), gb);
  hlayout->addWidget(label);
  label->setFixedSize(120,25);

  colorBtn = new KColorButton( bgColor, gb );
  connect( colorBtn, SIGNAL( changed( const QColor & ) ),
	   SLOT( slotBgColorChanged( const QColor & ) ) );
  hlayout->addWidget(colorBtn);
  colorBtn->setFixedSize(80,25);

  hlayout = new QHBoxLayout(5);
  vlayout->addLayout(hlayout);
  label = new QLabel( klocale->translate("Text Color:"), gb );
  hlayout->addWidget(label);
  label->setFixedSize(120, 25);
  colorBtn = new KColorButton( textColor, gb );
  connect( colorBtn, SIGNAL( changed( const QColor & ) ),
	   SLOT( slotTextColorChanged( const QColor & ) ) );
  hlayout->addWidget(colorBtn);
  colorBtn->setFixedSize(80,25);

  hlayout = new QHBoxLayout(5);
  vlayout->addLayout(hlayout);
  label = new QLabel( klocale->translate("Info Color:"), gb );
  hlayout->addWidget(label);
  label->setFixedSize(120, 25);

  colorBtn = new KColorButton( infoColor, gb );
  connect( colorBtn, SIGNAL( changed( const QColor & ) ),
	   SLOT( slotInfoColorChanged( const QColor & ) ) );
  hlayout->addWidget(colorBtn);
  colorBtn->setFixedSize(80,25);

  hlayout = new QHBoxLayout(5);
  vlayout->addLayout(hlayout);
  label = new QLabel( klocale->translate("Message Color:"), gb );
  hlayout->addWidget(label);
  label->setFixedSize(120, 25);

  colorBtn = new KColorButton( msgColor, gb );
  connect( colorBtn, SIGNAL( changed( const QColor & ) ),
	   SLOT( slotMsgColorChanged( const QColor & ) ) );
  hlayout->addWidget(colorBtn);
  colorBtn->setFixedSize(80,25);

  hlayout = new QHBoxLayout(5);
  vlayout->addLayout(hlayout);
  label = new QLabel( klocale->translate("Error Color:"), gb );
  hlayout->addWidget(label);
  label->setFixedSize(120, 25);

  colorBtn = new KColorButton( errorColor, gb );
  connect( colorBtn, SIGNAL( changed( const QColor & ) ),
	   SLOT( slotErrorColorChanged( const QColor & ) ) );
  hlayout->addWidget(colorBtn);
  colorBtn->setFixedSize(80,25);

  hlayout = new QHBoxLayout(5);
  vlayout->addLayout(hlayout);
  label = new QLabel( klocale->translate("Action Color:"), gb );
  hlayout->addWidget(label);
  label->setFixedSize(120, 25);

  colorBtn = new KColorButton( actionColor, gb );
  connect( colorBtn, SIGNAL( changed( const QColor & ) ),
	   SLOT( slotActionColorChanged( const QColor & ) ) );
  hlayout->addWidget(colorBtn);
  colorBtn->setFixedSize(80,25);

  vlayout->activate();
  lVerLayout->addStretch(10);
  horLayout->addStretch(10);
  horLayout->activate();
  show();
}

ColorOpt::~ColorOpt()
{
#ifdef EDEBUG
  cout << "Destruktor ColorOpt" << endl;
#endif
}

void ColorOpt::readOptions()
{
  KConfig *config = ircapp->getConfig();
  config->setGroup( "kEirc Option" );
  
  bgColor    = config->readColorEntry( "BgColor",    &white );
  textColor  = config->readColorEntry( "TextColor",  &black );
  infoColor  = config->readColorEntry( "InfoColor",  &blue  );
  msgColor   = config->readColorEntry( "MsgColor",   &red   );
  actionColor= config->readColorEntry( "ActionColor",&red   );
  errorColor = config->readColorEntry( "ErrorColor", &red   );
}

void ColorOpt::slotBgColorChanged( const QColor &col )
{
  if ( bgColor != col ){
     bgColor = col;
     ircapp->writeEntry( "BgColor",   bgColor );
     emit colorsChanged( bgColor, TYPE_BG);
  }
}

void ColorOpt::slotTextColorChanged( const QColor &col )
{
  if ( textColor != col ){
     textColor = col;
     ircapp->writeEntry( "TextColor",   textColor );
     emit colorsChanged( textColor, TYPE_TEXT);
  }
}
   
void ColorOpt::slotInfoColorChanged( const QColor &col )
{
  if ( infoColor != col ){
     infoColor = col;
     ircapp->writeEntry( "InfoColor",   infoColor );
     emit colorsChanged( infoColor, TYPE_INFO);
  }
}

void ColorOpt::slotMsgColorChanged( const QColor &col )
{
  if ( msgColor != col ){
     msgColor = col;
     ircapp->writeEntry( "MsgColor",   msgColor );
     emit colorsChanged( msgColor, TYPE_MSG);
  }
}

void ColorOpt::slotActionColorChanged( const QColor &col )
{
  if ( actionColor != col ){
     actionColor = col;
     ircapp->writeEntry( "ActionColor",   actionColor );
     emit colorsChanged( actionColor, TYPE_ACTION);
  }
}

void ColorOpt::slotErrorColorChanged( const QColor &col )
{
  if ( errorColor != col ){
     errorColor = col;
     ircapp->writeEntry( "ErrorColor",   errorColor );
     emit colorsChanged( errorColor, TYPE_ERROR);
  }
}




