#include <iostream.h>
#include <stdlib.h>
#include <qprogdlg.h> 
#include "kprocess.h"
#include "kconfig.h"
#include "ircApp.h"

QString IrcApp::home      = (const char*)getenv("HOME");
QString IrcApp::keircHome = (const char*)getenv("HOME")+QString("/.keirc");;
QString IrcApp::entryGroup= "kEirc Option";

IrcApp::IrcApp(int& argc, char** argv):
  KApplication(argc, argv)
{
  localHost = "localhost";
  getConfig()->setGroup(entryGroup);
  entryList = new QDict<char>(40, false, true);
  entryList->setAutoDelete(true);
  webServer = new WebServer;
}

IrcApp::~IrcApp()
{
#ifdef EDEBUG
  cout << "Destruktor IrcApp"<<endl;
#endif
  delete entryList;
  delete webServer;
}

// PathDefault relativ zu homedir
QString IrcApp::readPathEntry(const char* Entry, const char* PathDefault)
{
  getConfig()->setGroup(entryGroup);
  QString path = getConfig()->readEntry(Entry);
  if (path.isEmpty()){
     path=home+"/.keirc/"+QString(PathDefault);
     getConfig()->writeEntry(Entry, path);
     getConfig()->sync();
  }
  return path;
}

QString IrcApp::readListEntry(const char* Entry, const char* Default)
{
  QString value = entryList->find(Entry);
  if (value.isEmpty())
     value = Default;
  return value;
}

void IrcApp::writeListEntry(const char* Entry, const char* Value)
{
  if (entryList->find(Entry))
     entryList->replace(Entry, qstrdup(Value));
  else
     entryList->insert(Entry, qstrdup(Value));
  writeEntry(Entry, Value);
}

int IrcApp::readListEntry(const char* Entry, int Default)
{
  int res=Default;
  QString value = entryList->find(Entry);
  if (!value.isEmpty())
     res = value.toInt();
  return res;
}

void IrcApp::writeListEntry(const char* Entry, int Value)
{
  QString vs;
  vs.setNum(Value);
  writeListEntry(Entry, vs);
}

QString IrcApp::readEntry(const char* Entry, const char* Default)
{
  getConfig()->setGroup(entryGroup);
  return getConfig()->readEntry(Entry, Default);
}

int IrcApp::readNumEntry(const char* Entry, int Default)
{
  getConfig()->setGroup(entryGroup);
  return getConfig()->readNumEntry(Entry, Default);
}

void IrcApp::writeEntry(const char* Entry, const char* Value)
{
  getConfig()->setGroup(entryGroup);
  getConfig()->writeEntry(Entry, Value);
  getConfig()->sync();
}

void IrcApp::writeEntry(const char* Entry, int Value)
{
  getConfig()->setGroup(entryGroup);
  getConfig()->writeEntry(Entry, Value);
  getConfig()->sync();
}

void IrcApp::writeEntry(const char* Entry, const QColor& clr)
{
  getConfig()->setGroup(entryGroup);
  getConfig()->writeEntry(Entry, clr);
  getConfig()->sync();
}

int IrcApp::browseInit()
{
  QString val=readEntry("OnBrowser", "Netscape");
  if (!stricmp(val, "Netscape")){
     int status=remoteNetscape.Init();
     if (!status){
        int totalStep=20;
        QProgressDialog progress( "Wait...execute netscape", "Abort", totalStep, NULL);
	progress.setCaption("Execute netscape");
	progress.setProgress(1);
        KProcess proc;
	proc.setExecutable("netscape");
	proc.start(KProcess::DontCare);
	for (int i=1; i<totalStep && !status; i++) {
	    progress.setProgress(i);
	    if (progress.wasCancelled())
	       break;
	    status=remoteNetscape.Init();
	}
     }
     return status;
  }
  return 0;
}

void IrcApp::browse(const char* cmd)
{
  setOverrideCursor(waitCursor);
  int status = browseInit();

  if (status){
     QString val=readEntry("OnBrowser", "Netscape");
     if (!stricmp(val, "Netscape")){
        remoteNetscape.commands(cmd);
     }
  }
  restoreOverrideCursor();
}

void IrcApp::browse(const char* Cmd, const char* Arg)
{
  setOverrideCursor(waitCursor);
  int status = browseInit();

  if (status){
     QString val=readEntry("OnBrowser", "Netscape");
     if (!stricmp(val, "Netscape")){
        QString cmd=QString(Cmd)+"("+QString(Arg)+")";
	remoteNetscape.commands(cmd);
     }
  }
  restoreOverrideCursor();
}



