#include <iostream.h>
#include <qstring.h>
#include "ircLine.moc"
#include "ircLine.h"

IrcLine::IrcLine(QWidget* parent, const char* name, int buff,  bool clean) : 
  QLineEdit(parent, name)
{
#ifdef EDEBUG
  cout << "Konstruktor IrcLine..."<<endl;
#endif
   autoClean = clean;
   history   = new RingList<char>(buff);
   history->add(qstrdup(""));
   fontSize  = 2;
   fontName  = "times";
   textColor = QColor(black);
   QFont f(fontName, 10+fontSize);
   setFont(f);
   connect(this, SIGNAL(returnPressed()), SLOT(slotTextReturn()));
}

IrcLine::~IrcLine()
{
#ifdef EDEBUG
  cout << "Destruktor IrcLine" << endl;
#endif
  delete history;
}

void IrcLine::mousePressEvent(QMouseEvent* e)
{
  QLineEdit::mousePressEvent(e);
  if (e->button()==RightButton){
    QPoint pos=mapToGlobal(e->pos());
    emit signMouseRightPress(pos);
  }
}

void IrcLine::slotTextReturn()
{
   QString t = text();
   if (t.isNull() || t.isEmpty())
      return;
   else{
      history->addLast(qstrdup(t));
      emit signReturn(t);
      history->add(qstrdup(""));
      if (autoClean)
	 setText("");
   }
}

void IrcLine::keyPressEvent(QKeyEvent* e)
{
  int k = e->key();

  if (e->state() & ControlButton){
     QString txt = text();
     switch (k) {
     case Key_B:
       txt += 0x02;
       setText(txt);
       return;
       break;
     case Key_U:
       txt += 0x1F;
       setText(txt);
       return;
       break;
     case Key_I:
       txt += 0x16;
       setText(txt);
       return;
       break;
     case Key_K:
       txt += 0x03;
       setText(txt);
       return;
       break;
     case Key_O:
       txt += 0x0F;
       setText(txt);
       return;
       break;
     }
  }

  if (k==Key_Down){
      QString s=history->forward();
      setText(s);
      return;
  }
  else if (k==Key_Up){
      setText(history->back());
      return;
  }
  QLineEdit::keyPressEvent(e);
  setFocus();
}

