#include <iostream.h>
#include "kiconloader.h"
#include "lineDialog.h"
#include "ircView.h"
#include "ircTopWidget.h"
#include "ircTopWidget.moc"
#include "kEirc.h"

IrcTopWidget::IrcTopWidget(IrcClient* Client, QWidget* parent, const char *name) : 
  KTopLevelWidget(name)
{
   if (name)
      this->name=name;
   else
      name="IrcTopWidget";
   client  = Client;
   KEIrc* keirc = (KEIrc*)parent;
   ircView = keirc->ircView();
   window  = keirc->windowList();
   qApp->installEventFilter(this);

   idSbNick   = 0;
   idSbStatus = 1;
   idSbLagging= 2;
   idSbLive   = 3;

   idTbCounter = 0;
   idTbDCCChat = idTbCounter++;
   idTbDCCSend = idTbCounter++;
   idTbWindow  = idTbCounter++;
   
   createStatusbar();
   createMenu();
   createToolbar();

   setStatusBar(statusbar);
   addToolBar  (toolbar);

   setFrameBorderWidth(0);
   
   enableStatusBar(KStatusBar::Show);
   enableToolBar  (KToolBar::Show);

   setMinimumSize( 550, 300);

   // Connection
   connect(client,
	   SIGNAL(signConnected(IrcServer*)),
	   this,
	   SLOT  (slotConnected(IrcServer*))
	   );
   connect(client,
	   SIGNAL(signDisconnected(IrcServer*)),
	   this,
	   SLOT  (slotDisconnected(IrcServer*))
	   );
   connect(client,
	   SIGNAL(signCurNick(const char*)),
	   this,
	   SLOT  (slotCurNick(const char*)));
   connect(client, 
	   SIGNAL(signCurServer(IrcServer*)), 
	   this,   
	   SLOT  (slotCurServer(IrcServer*)));

   connect(client,
	   SIGNAL(signMinTimeout(IrcCore*)),
	   this,
	   SLOT  (slotLiveStatus(IrcCore*)));
   connect(client,
	   SIGNAL(signLagging(const char*)),
	   this,
	   SLOT  (slotLagging(const char*)));

   setCaption(this->name);
   QString geometry = ircapp->readEntry( this->name+"Geometry", "");
   if (!geometry.isEmpty()){
      int width, height;
      sscanf( geometry, "%dx%d", &width, &height );
      resize( width, height );
   }
   else
      resize(width(), height());
}


IrcTopWidget::~IrcTopWidget()
{
#ifdef EDEBUG
  cout << "Destruktor IrcTopWidget" << endl;
#endif
  delete toolbar;
  delete statusbar;
#ifdef EDEBUG
  cout << "End Destruktor IrcTopWidget" << endl;
#endif
}

void IrcTopWidget::createMenu()
{
   menu.setAutoDelete (false);

   menu.insert("window", new QPopupMenu);
   menu["window"]->setCheckable( true );
   menu["window"]->insertItem("k&Eirc Window");
   menu["window"]->insertItem("k&Channel Window"); 
   menu["window"]->insertItem("k&PrivMsg Window"); 
   menu["window"]->insertItem("k&DCC Window"); 
   connect(menu["window"], SIGNAL(activated(int)),
	   this,           SLOT  (slotWindow(int)));

}

void IrcTopWidget::createToolbar()
{
   QPixmap pixmap;
   KIconLoader* loader = ircapp->getIconLoader();

   toolbar = new KToolBar(this);
   CHECK_PTR(toolbar);

   // Einstellen
   toolbar->setFullWidth(TRUE);

   pixmap = loader->loadIcon("dccchat.xpm");
   toolbar->insertButton(pixmap, idTbDCCChat, SIGNAL(clicked()), 
			 this, SLOT(slotDCCChat()),
			 TRUE, "DCC Chat");

   pixmap = loader->loadIcon("dccsend.xpm");
   toolbar->insertButton(pixmap, idTbDCCSend, SIGNAL(clicked()), 
			 this, SLOT(slotDCCSend()),
			 TRUE, "DCC Send");

   toolbar->insertSeparator();
   toolbar->insertSeparator();

   int id = idTbCounter++;
   pixmap = ircapp->getIconLoader()->loadIcon("ctcpsound.xpm");
   toolbar->insertButton(pixmap, id, SIGNAL(clicked()), 
			 this, SLOT(slotCtcpSound()),
			 TRUE, "CTCP Sound");

   toolbar->insertSeparator();
   toolbar->insertSeparator();

   id = idTbCounter++;
   pixmap = ircapp->getIconLoader()->loadIcon("optpref.xpm");
   toolbar->insertButton(pixmap, id, SIGNAL(clicked()), 
			 this, SLOT(slotOptPref()),
			 TRUE, "Option Preferences");

   id = idTbCounter++;
   pixmap = ircapp->getIconLoader()->loadIcon("optprivmsg.xpm");
   toolbar->insertButton(pixmap, id, SIGNAL(clicked()), 
			 this, SLOT(slotOptPrivMsg()),
			 TRUE, "Option Private message");

   id = idTbCounter++;
   pixmap = ircapp->getIconLoader()->loadIcon("optchannel.xpm");
   toolbar->insertButton(pixmap, id, SIGNAL(clicked()), 
			 this, SLOT(slotOptChannel()),
			 TRUE, "Option Channel");

   id = idTbCounter++;
   pixmap = ircapp->getIconLoader()->loadIcon("optdcc.xpm");
   toolbar->insertButton(pixmap, id, SIGNAL(clicked()), 
			 this, SLOT(slotOptDCC()),
			 TRUE, "Option DCC");

   pixmap = loader->loadIcon("windows.xpm");
   toolbar->insertButton(pixmap, idTbWindow, SIGNAL(pressed()), 
			 this, SLOT(slotWindowPopup()),
			 TRUE, "Window");
   
   toolbar->alignItemRight (idTbWindow);
   toolbar->setBarPos(KToolBar::Top);
}

void IrcTopWidget::createStatusbar()
{
   statusbar = new KStatusBar(this);
   QFont f("times",12,QFont::Bold);
   statusbar->setFont(f);
   statusbar->insertItem("Nick: 1234567891234567", idSbNick);
   statusbar->insertItem("Status: Not connect", idSbStatus);
   statusbar->insertItem("Lagging: 1234567891234", idSbLagging);
   statusbar->insertItem  ("      ", idSbLive);
   statusbar->setAlignment(idSbLive, AlignRight);

   statusbar->changeItem("Nick:                 ", idSbNick);
   statusbar->changeItem("Lagging:              ", idSbLagging);
   statusbar->changeItem("00:00", idSbLive);
}

void IrcTopWidget::resizeEvent( QResizeEvent * )
{
   QString Geometry;
   Geometry.sprintf("%dx%d", geometry().width(), geometry().height());
   ircapp->writeEntry(name+"Geometry", Geometry);
   toolbar->updateRects();
   updateRects();
}


void IrcTopWidget::closeEvent (QCloseEvent *)
{
  hide();
}

void IrcTopWidget::slotConnected(IrcServer* is)
{
  statusbar->changeItem("Status: Connected", idSbStatus);
  QString st="";
  st += name+" "+is->Host()+":"+is->Port();
  setCaption(st);
}

void IrcTopWidget::slotDisconnected(IrcServer*)
{
  statusbar->changeItem("Status: Not connect", idSbStatus);
  statusbar->changeItem("Lagging: ", idSbLagging);
  statusbar->changeItem("00:00", idSbLive);
  setCaption(name);
}

void IrcTopWidget::slotCurNick(const char* nick)
{
  statusbar->changeItem(QString("Nick: ")+nick, idSbNick);
}

void IrcTopWidget::slotCurServer(IrcServer* is)
{
  if (client->state>0)
     return;
  if (client->getSocket().getSocket()>0){
     QString st="";
     st += name+" "+is->Host()+":"+is->Port();
     setCaption(st);
  }
}

void IrcTopWidget::slotWindowPopup()
{
  toolbar->setButton(idTbWindow, false);
  QRect  r = toolbar->getButton(idTbWindow)->geometry();
  QPoint p = r.bottomLeft(); 
  menu["window"]->popup(mapToGlobal(p));
}

void IrcTopWidget::slotWindow(int id)
{
  if (menu["window"]->isItemChecked(id))
     return;
  int idx = menu["window"]->indexOf(id);
  QWidget* w=NULL;
  switch(idx){
  case 0:
    w = window->find("kEirc");
    break;
  case 1:
    w = window->find("kChannel");
    break;
  case 2:
    w = window->find("kPrivMsg");
    break;
  case 3:
    w = window->find("kDCC");
    break;
  }
  if (w){
    if (w->isVisible()){
       QPoint pos = w->pos();
       w->hide();
       w->move(pos);
    }
    w->show();
  }
}


void IrcTopWidget::slotLiveStatus(IrcCore* ic)
{
  ulong minutes = ic->getMinutes();
  int hour  = minutes/60;
  int min   = minutes%60;
  QString ms;
  ms.sprintf("%02d:%02d", hour, min);
  statusbar->changeItem(ms, idSbLive);
}


void IrcTopWidget::slotLagging(const char* l)
{
  QString ms="Lagging : " +QString(l)+" seconds";
  statusbar->changeItem(ms, idSbLagging);
}

void IrcTopWidget::slotDCCChat()
{
  SingleDialog dlg("To :","","Type nick");
  if (dlg.exec()){
     QString nick=dlg.getText(TRUE);
     if (!nick.isEmpty()){
        QString s="DCC CHAT "+nick;
	client->slotCommand(nick, "/DCC CHAT "+nick);
     }
  }
}

void IrcTopWidget::slotDCCSend()
{
  client->dccFileDlg->show();
}

void IrcTopWidget::slotCtcpSound()
{
  client->mediaDlg->show();
}

void IrcTopWidget::slotOptPref()
{
  client->optPrefDlg->show();
}

void IrcTopWidget::slotOptPrivMsg()
{
  client->optPrefDlg->show("PrivMsg");
}

void IrcTopWidget::slotOptChannel()
{
  client->optPrefDlg->show("Channel");
}

void IrcTopWidget::slotOptDCC()
{
  client->optPrefDlg->show("DCC");
}
