#include <iostream.h>
#include "kiconloader.h"
#include "ircView.h"
#include "ircDialog.h"
#include "lineDialog.h"
#include "dccFileDialog.h"
#include "kEirc.h"
#include "kEirc.moc"

QList<KEIrc> KEIrc::keircList;
int          KEIrc::nClient=0;

KEIrc::KEIrc(const char *name) : KTopLevelWidget(name)
{
   nClient++;
   name = "kEirc";
   idTbConnect    = 0;
   idTbDisconnect = 1;
   idTbCombo      = 2;
   idTbDCCChat    = 3;
   idTbDCCSend    = 4;
   idTbCtcpSound  = 5;
   idTbWebServer  = 6;
   idSbNick   = 0;
   idSbStatus = 1;
   idSbMode   = 2;
   idSbLagging= 3;
   idSbLive   = 4;
   client = new IrcClient();

   setFocusPolicy(QWidget::StrongFocus);
   qApp->installEventFilter(this);

   createMenu     ();
   createToolbar  ();
   createStatusbar();

   setMenu        (menubar);
   menubar->setFrameStyle(QFrame::WinPanel | QFrame::Raised);
   menubar->setLineWidth(2);
   menubar->setFont(QFont("Times", 13, QFont::Bold, TRUE));

   setStatusBar   (statusbar);
   addToolBar     (toolbar);

   readConfig();

   setFrameBorderWidth(0);
   setView(view=new IrcView(this->client, this));

   menubar->show();

   if (showStatusbar)
      enableStatusBar(KStatusBar::Show);
   else
      enableStatusBar(KStatusBar::Hide);

   if (showToolbar) 
      enableToolBar(KToolBar::Show);
   else
      enableToolBar(KToolBar::Hide);

   keircList.setAutoDelete(false);
   keircList.append( this );

   setMinimumSize( 550, 300 );

   menu["options"]->setItemChecked(idOptChannel,   showChannel);
   menu["options"]->setItemChecked(idOptPrivMsg,   showPrivMsg);
   menu["options"]->setItemChecked(idOptDCC,       showDCC);
   menu["options"]->setItemChecked(idOptToolbar,   showToolbar);
   menu["options"]->setItemChecked(idOptStatusbar, showStatusbar);

   QString geometry = ircapp->readEntry(this->name+"Geometry", "" );
   if (!geometry.isEmpty()){
      int width, height;
      sscanf( geometry, "%dx%d", &width, &height );
      resize( width, height );
   }
   // Connection
   connect(client,
	   SIGNAL(signConnected(IrcServer*)),
	   this,
	   SLOT  (slotConnected(IrcServer*))
	   );
   connect(client,
	   SIGNAL(signDisconnected(IrcServer*)),
	   this,
	   SLOT  (slotDisconnected(IrcServer*))
	   );
   connect(client,
	   SIGNAL(signCurNick(const char*)),
	   this,
	   SLOT  (slotCurNick(const char*)));
   connect(client, 
	   SIGNAL(signAddServer(IrcServer*)), 
	   this,   
	   SLOT  (slotComboAddServer(IrcServer*)));
   connect(client, 
	   SIGNAL(signCurServer(IrcServer*)), 
	   this,   
	   SLOT  (slotCurServer(IrcServer*)));
   connect(client, 
	   SIGNAL(signDelServer(const char*)), 
	   this,   
	   SLOT  (slotComboDelServer(const char*)));
   connect(client, 
	   SIGNAL(signMode(const char*)), 
	   this,   
	   SLOT  (slotMode(const char*)));

   connect(client,
	   SIGNAL(signMinTimeout(IrcCore*)),
	   this,
	   SLOT  (slotLiveStatus(IrcCore*)));
   connect(client,
	   SIGNAL(signLagging(const char*)),
	   this,
	   SLOT  (slotLagging(const char*)));

   connect(client,
	   SIGNAL(signWriteOnActive(int , const char*, bool)),
	   this,
	   SLOT  (slotWriteOnActive(int , const char*, bool)));

   resize(width(), height());
   setCaption("kEirc");

   window.setAutoDelete(false);
   window.insert("kEirc", this);
   window.insert("kChannel", kChannel = new KChannel(this->client, this, "kEirc Channel"));
   window.insert("kPrivMsg", kPrivMsg = new KPrivMsg(this->client, this, "kEirc PrivMsg"));
   window.insert("kDCC",     kDCC     = new KDCC    (this->client, this, "kEirc DCC"));
   show();
   if (!showChannel)
      kChannel->hide();
   else
      kChannel->show();
   if (!showPrivMsg)
      kPrivMsg->hide();
   else
      kPrivMsg->show();
   if (!showDCC)
      kDCC->hide();
   else
      kDCC->show();
   client->initAll();
   QString s;
   if ((s=ircapp->readEntry("WebServerAutoStart", "No")) && s.find("Yes", 0, false)==0)
      ircapp->webServer->slotOpen();

   if ((s=ircapp->readEntry("AutoConnect", "No")) && s.find("Yes", 0, false)==0)
      slotFileConnect();
#ifdef EDEBUG
    cout << "Konstruktor KEIrc...Ok" << endl;
#endif 
}


KEIrc::~KEIrc()
{
#ifdef EDEBUG
    cout << "Destruktor KEIrc" << endl;
#endif 
    keircList.removeRef(this);
    nClient--;
    delete menubar;
    delete toolbar;
    delete statusbar;
    delete kChannel;
    delete kPrivMsg;
    delete kDCC;
    delete client;
}

void KEIrc::createMenu()
{
   menu.setAutoDelete (false);

   menu.insert("file",    new QPopupMenu);
   idFileNew     = menu["file"]->insertItem("&New Client",     this, SLOT(slotFileNew()),     CTRL+Key_N);
   menu["file"]->insertSeparator();
   idFileConnect    = menu["file"]->insertItem("Connect Server",     this, SLOT(slotFileConnect()));
   idFileDisconnect = menu["file"]->insertItem("Disconnect Server",  this, SLOT(slotFileDisconnect()));
   menu["file"]->insertSeparator();
   idFileClose   = menu["file"]->insertItem("&Close Client",   this, SLOT(slotFileClose()),   CTRL+Key_C);
   idFileExit    = menu["file"]->insertItem("E&xit",           this, SLOT(slotFileExit()),    CTRL+Key_X);

   menu.insert("options", new QPopupMenu);
   menu["options"]->setCheckable( true );

   idOptPref      = menu["options"]->insertItem("Pre&ference", this, SLOT(slotOptionsPref()));
   menu["options"]->insertSeparator();
   idOptChannel   = menu["options"]->insertItem("Show &Channel Window", this, SLOT(slotOptionsChannel()));
   idOptPrivMsg   = menu["options"]->insertItem("Show &PrivMsg Window", this, SLOT(slotOptionsPrivMsg()));
   idOptDCC       = menu["options"]->insertItem("Show &DCC Window",     this, SLOT(slotOptionsDCC()));
   menu["options"]->insertSeparator();
   idOptToolbar   = menu["options"]->insertItem("Show &Toolbar",   this, SLOT(slotOptionsToolbar  ()));
   idOptStatusbar = menu["options"]->insertItem("Show Status&bar", this, SLOT(slotOptionsStatusbar()));
   menu["options"]->insertSeparator();
   idOptSave      = menu["options"]->insertItem("&Save Options",   this, SLOT(slotOptionsSave()));

   menu.insert("window", new QPopupMenu);
   menu["window"]->setCheckable( true );
   int idEirc;
   idEirc = menu["window"]->insertItem("k&Eirc Window"); 
   menu["window"]->setItemChecked(idEirc,  1);
   menu["window"]->insertItem("k&Channel Window"); 
   menu["window"]->insertItem("k&PrivMsg Window"); 
   menu["window"]->insertItem("k&DCC Window"); 
   connect(menu["window"], SIGNAL(activated(int)),
	   this,           SLOT  (slotWindow(int)));

   menu.insert("help", new QPopupMenu);
   menu["help"]->insertItem("&Using keirc Help", this, SLOT(slotHelpUsing())); 
   menu["help"]->insertSeparator();
   menu["help"]->insertItem("&About", this, SLOT(slotHelpAbout()) );

   menubar = new KMenuBar( this );
   CHECK_PTR(menubar);
   menubar->insertItem("&File",    menu["file"]);
   menubar->insertItem("&Options", menu["options"]);
   menubar->insertItem("&Windows", menu["window"]);
   menubar->insertSeparator();
   menubar->insertItem("&Help",    menu["help"]);
}


void KEIrc::createToolbar()
{
   QPixmap pixmap;

   toolbar = new KToolBar(this, "ircToolBar", 30);
   CHECK_PTR(toolbar);

   // Einstellen
   toolbar->setFullWidth(TRUE);

   pixmap = ircapp->getIconLoader()->loadIcon("connected.xpm");
   toolbar->insertButton(pixmap, idTbConnect, SIGNAL(clicked()), 
			 this, SLOT(slotFileConnect()),
			 TRUE, "Connect to server");

   pixmap = ircapp->getIconLoader()->loadIcon("disconnected.xpm");
   toolbar->insertButton(pixmap, idTbDisconnect, SIGNAL(clicked()), 
			 this, SLOT(slotFileDisconnect()),
			 TRUE, "Disconnect server");

   toolbar->insertSeparator();
   toolbar->insertSeparator();

   pixmap = ircapp->getIconLoader()->loadIcon("dccchat.xpm");
   toolbar->insertButton(pixmap, idTbDCCChat, SIGNAL(clicked()), 
			 this, SLOT(slotDCCChat()),
			 TRUE, "DCC Chat");

   pixmap = ircapp->getIconLoader()->loadIcon("dccsend.xpm");
   toolbar->insertButton(pixmap, idTbDCCSend, SIGNAL(clicked()), 
			 this, SLOT(slotDCCSend()),
			 TRUE, "DCC Send");

   toolbar->insertSeparator();
   toolbar->insertSeparator();

   pixmap = ircapp->getIconLoader()->loadIcon("ctcpsound.xpm");
   toolbar->insertButton(pixmap, idTbCtcpSound, SIGNAL(clicked()), 
			 this, SLOT(slotCtcpSound()),
			 TRUE, "CTCP Sound");

   toolbar->insertSeparator();
   toolbar->insertSeparator();

   int idTbCounter=7;
   int id = idTbCounter++;
   pixmap = ircapp->getIconLoader()->loadIcon("optpref.xpm");
   toolbar->insertButton(pixmap, id, SIGNAL(clicked()), 
			 this, SLOT(slotOptPref()),
			 TRUE, "Option Preferences");

   toolbar->insertSeparator();
   toolbar->insertSeparator();

   pixmap = ircapp->getIconLoader()->loadIcon("webserver.xpm");
   toolbar->insertButton(pixmap, idTbWebServer, SIGNAL(clicked()), 
			 ircapp->webServer, SLOT(slotOpen()),
			 TRUE, "Web Server");

   toolbar->insertCombo ("", idTbCombo, TRUE, 
			 SIGNAL(activated(const char *)), 
			 this, SLOT(slotComboServer(const char *)), 
			 TRUE, "Server List", 250);

   toolbar->clearCombo(idTbCombo);
   QDictIterator<IrcServer> it(*client->server);
   QString st;
   for(;it.current();++it){
      IrcServer* is=it.current();
      st=is->Host()+":"+is->Port();
      toolbar->insertComboItem(idTbCombo, st, -1);
   }
   toolbar->alignItemRight (idTbCombo);
   toolbar->setBarPos(KToolBar::Top);
}


void KEIrc::createStatusbar()
{
   statusbar = new KStatusBar(this);
   statusbar->insertItem("Nick: 1234567891234567", idSbNick);
   statusbar->insertItem("Status: Not connect", idSbStatus);
   statusbar->insertItem("Mode: 123456", idSbMode);
   statusbar->insertItem("Lagging: 1234567891234", idSbLagging);
   statusbar->insertItem  ("      ", idSbLive);
   statusbar->setAlignment(idSbLive, AlignRight);

   statusbar->changeItem("Nick:                 ", idSbNick);
   statusbar->changeItem("Mode:       ", idSbMode);
   statusbar->changeItem("Lagging:              ", idSbLagging);
   statusbar->changeItem("00:00", idSbLive);
}

void KEIrc::readConfig()
{
   KConfig* config = ircapp->getConfig();
   QString  cfgString;;
   config->setGroup("kEirc Option");

   cfgString = config->readEntry( "ShowChannel" );
   if (!cfgString.isEmpty() && cfgString.find( "No", 0, false ) == 0 )
       showChannel = false;
   else
       showChannel = true;

   cfgString = config->readEntry( "ShowPrivMsg" );
   if (!cfgString.isEmpty() && cfgString.find( "No", 0, false ) == 0 )
       showPrivMsg = false;
   else
       showPrivMsg = true;

   cfgString = config->readEntry( "ShowDCC" );
   if (!cfgString.isEmpty() && cfgString.find( "No", 0, false ) == 0 )
       showDCC = false;
   else
       showDCC = true;

   cfgString = config->readEntry( "ShowToolBar" );
   if (!cfgString.isEmpty() && cfgString.find( "No", 0, false ) == 0 )
       showToolbar = false;
   else
       showToolbar = true;

   cfgString = config->readEntry( "ToolBarPos" );
   if (cfgString.isEmpty() )
      toolBar()->setBarPos(KToolBar::Top);
   else if ("Top" == cfgString) 
      toolBar()->setBarPos(KToolBar::Top);
   else if ("Bottom" == cfgString)
      toolBar()->setBarPos(KToolBar::Bottom);
   else if ("Left" == cfgString)
      toolBar()->setBarPos(KToolBar::Left);
   else if ("Right" == cfgString)
      toolBar()->setBarPos(KToolBar::Right);
   else
      toolBar()->setBarPos(KToolBar::Top);

   cfgString = config->readEntry( "ShowStatusBar" );
   if (!cfgString.isEmpty() && cfgString.find( "No", 0, false ) == 0 )
       showStatusbar = false;
   else
       showStatusbar = true;
}

void KEIrc::enableMenuItems()
{
   menu["file"]->setItemEnabled(idFileConnect, 1);
   updateRects();
}


void KEIrc::resizeEvent( QResizeEvent * )
{
   QString Geometry;
   Geometry.sprintf("%dx%d", geometry().width(), geometry().height());
   ircapp->writeEntry(name+"Geometry", Geometry);
   view->setGeometry(0,0,view_right, view_bottom);
   updateRects();
}

void KEIrc::closeEvent (QCloseEvent *)
{
    delete this;
    if (keircList.count()<1)
      //if (nClient<1)
       kapp->quit();
}

void KEIrc::slotFileNew()
{
   KEIrc* newItem;

   newItem = new KEIrc("kEirc");
   CHECK_PTR(newItem);
   newItem->resize(size());
   newItem->show();
 }

void KEIrc::slotFileConnect()
{  
  client->slotConnect(NULL);
}

void KEIrc::slotFileDisconnect()
{
  client->slotDisconnected();
}

void KEIrc::slotFileClose()
{
   close();
}

void KEIrc::slotFileExit()
{
  KEIrc* k;
  for (k=keircList.first();k=keircList.current();)
      delete k;
  kapp->quit();
}

void KEIrc::slotOptionsPref()
{
  client->optPrefDlg->show();
}
  

void KEIrc::slotOptionsChannel()
{
   if (showChannel){
      window["kChannel"]->hide();
      showChannel = false;
   }
   else{
      window["kChannel"]->show();
      showChannel = true;
   }
  	
   menu["options"]->setItemChecked(idOptChannel, showChannel);
   updateRects();
}

void KEIrc::slotOptionsPrivMsg()
{
   if (showPrivMsg){
      window["kPrivMsg"]->hide();
      showPrivMsg = false;
   }
   else{
      window["kPrivMsg"]->show();
      showPrivMsg = true;
   }
  	
   menu["options"]->setItemChecked(idOptPrivMsg, showPrivMsg);
   updateRects();
}

void KEIrc::slotOptionsDCC()
{
   if (showDCC){
      window["kDCC"]->hide();
      showDCC = false;
   }
   else{
      window["kDCC"]->show();
      showDCC = true;
   }
  	
   menu["options"]->setItemChecked(idOptDCC, showDCC);
   updateRects();
}

void KEIrc::slotOptionsToolbar()
{
   if (showToolbar){
      enableToolBar(KToolBar::Hide);
      showToolbar = false;
   }
   else{
      enableToolBar(KToolBar::Show);
      showToolbar = true;
   }
  	
   menu["options"]->setItemChecked(idOptToolbar, showToolbar);
   updateRects();
}



void KEIrc::slotHelpUsing()
{
   ircapp->invokeHTMLHelp("keirc/keirc.html", "");
}

void KEIrc::slotHelpAbout()
{
   QMessageBox mb;
   mb.setText( "KEIRC \nVersion " + QString( KEIRC_VERSION ) +
	       "\n\nEko Bono <sj039su@uni-duisburg.de>" );
   mb.setCaption( "About keirc" );
   mb.show();
}


void KEIrc::slotOptionsStatusbar()
{
   showStatusbar = !showStatusbar;
   menu["options"]->setItemChecked(idOptStatusbar, showStatusbar); 
   enableStatusBar(KStatusBar::Toggle);
}


void KEIrc::slotOptionsSave()
{
   KConfig *config = kapp->getConfig();

   config->setGroup( "kEirc Option" ); 
   config->writeEntry( "ShowToolBar", showToolbar ? "Yes" : "No" );
   switch (toolbar->barPos()){
	case KToolBar::Top:
	   config->writeEntry( "ToolBarPos", "Top");
	   break;
	case KToolBar::Bottom:
	   config->writeEntry( "ToolBarPos", "Bottom");
	   break;
	case KToolBar::Left:
	   config->writeEntry( "ToolBarPos", "Left");
	   break;
	case KToolBar::Right:
	   config->writeEntry( "ToolBarPos", "Right");
	   break;
	default:
	   warning("KEIrc::slotOptionsSave: illegal default in case reached\n");
	   break;
   }
   config->writeEntry( "ShowStatusBar", showStatusbar ? "Yes" : "No" );
   config->writeEntry( "ShowChannel",   showChannel   ? "Yes" : "No" );
   config->writeEntry( "ShowPrivMsg",   showPrivMsg   ? "Yes" : "No" );
   config->writeEntry( "ShowDCC",       showDCC       ? "Yes" : "No" );
}

void KEIrc::slotWindow(int id)
{
  int idx = menu["window"]->indexOf(id);
  QWidget* w=NULL;
  switch(idx){
  case 1:
    w = window["kChannel"];
    break;
  case 2:
    w = window["kPrivMsg"];
    break;
  case 3:
    w = window["kDCC"];
    break;
  }
  if (w){
    if (w->isVisible()){
       QPoint pos = w->pos();
       w->hide();
       w->move(pos);
    }
    w->show();
  }
}

void KEIrc::slotComboServer(const char* txt)
{
  if (!txt)
     return;
  client->setCurServer(txt);
}

void KEIrc::slotComboAddServer(IrcServer* is)
{
  KToolBarCombo* cb=toolbar->getCombo(idTbCombo);
  QString st=is->Host()+":"+is->Port();
  cb->insertItem(st);
}

void KEIrc::slotComboCurServer(IrcServer* is)
{
  KToolBarCombo* cb=toolbar->getCombo(idTbCombo);
  for (uint i=0;i<(int)cb->count();i++){
      QString st=is->Host()+":"+is->Port();
      if (st==cb->text(i)){
	 cb->setCurrentItem(i);
	 return;
      }
  }
}



void KEIrc::slotComboDelServer(const char* s)
{
  KToolBarCombo* cb=toolbar->getCombo(idTbCombo);
  for (uint i=0;i<(int)cb->count();i++){
      QString st=s;
      if (st==cb->text(i)){
	 cb->removeItem(i);
	 return;
      }
  }
}

void KEIrc::slotConnected(IrcServer* is)
{
  ircapp->restoreOverrideCursor();
  QString str(4096);
  str = "Connected to port "+is->Port()+" of server "+is->Host();
  client->slotWriteMsg(TYPE_INFO|TYPE_IMG, str);
  str.sprintf("Your hostaddress is %s , your hostname is %s", 
	      client->getSocket().getLocalInetAddress().getHostAddress().data(),
	      client->getSocket().getLocalInetAddress().getHostName().data());;
  client->slotWriteMsg(TYPE_INFO|TYPE_IMG, str);
  
  menu["file"]->setItemEnabled(idFileConnect,    0);
  toolbar->setItemEnabled(idTbConnect,    0);
  menu["file"]->setItemEnabled(idFileDisconnect, 1);
  toolbar->setItemEnabled(idTbDisconnect, 1);
  statusbar->changeItem("Status: Connected", idSbStatus);
  QString st="";
  st += "kEirc "+client->curServer->Host()+":"+client->curServer->Port();
  setCaption(st);
}

void KEIrc::slotDisconnected(IrcServer* is)
{
  if (is && client->state){
     QString s="Disconnected from server "+is->Host()+" on port "+is->Port();
     client->slotWriteMsg(TYPE_INFO|TYPE_IMG, s);
  }

  menu["file"]->setItemEnabled(idFileConnect,    1);
  toolbar->setItemEnabled(idTbConnect,    1);
  if (client->state){
     menu["file"]->setItemEnabled(idFileDisconnect, 0);
     toolbar->setItemEnabled(idTbDisconnect, 0);
  }
  statusbar->changeItem("Status: Not connect", idSbStatus);
  statusbar->changeItem("Mode: ", idSbMode);
  statusbar->changeItem("Lagging: ", idSbLagging);
  statusbar->changeItem("00:00", idSbLive);
  setCaption("kEirc");
}

void KEIrc::slotCurNick(const char* nick)
{
  statusbar->changeItem("Nick: "+QString(nick), idSbNick);
}

void KEIrc::slotMode(const char* txt)
{
  QString st;
  st.sprintf("Mode: %s", txt);
  statusbar->changeItem(st, idSbMode);
}

void KEIrc::slotCurServer(IrcServer* is)
{
  if (client->state>0)
     return;
  slotComboCurServer(is);
  if (client->getSocket().getSocket()>0){
     QString st="";
     st += "kEirc "+is->Host()+":"+is->Port();
     setCaption(st);
  }
}


void KEIrc::slotLiveStatus(IrcCore* ic)
{
  ulong minutes = ic->getMinutes();
  int hour  = minutes/60;
  int min   = minutes%60;
  QString ms;
  ms.sprintf("%02d:%02d", hour, min);
  statusbar->changeItem(ms, idSbLive);
}

void KEIrc::slotLagging(const char* l)
{
  QString ms="Lagging : " +QString(l)+" seconds";
  statusbar->changeItem(ms, idSbLagging);
}

void KEIrc::slotWriteOnActive(int Type, const char* txt, bool Filter)
{
  QDictIterator<QWidget> it(window);
  QWidget* w;
  for (w=it.current();w=it.current();++it){
      QWidget* tw=w->topLevelWidget();
      if (tw->isActiveWindow()){
	 return;
      }
  }
  client->slotWriteMsg(Type, txt, Filter);
}

void KEIrc::slotDCCChat()
{
  SingleDialog dlg("To :","","Type nick");
  if (dlg.exec()==QDialog::Accepted){
     QString nick=dlg.getText(TRUE);
     if (!nick.isEmpty()){
        QString s="DCC CHAT "+nick;
	client->slotCommand(nick, "/DCC CHAT "+nick);
     }
  }
}

void KEIrc::slotDCCSend()
{
  client->dccFileDlg->show();
}

void KEIrc::slotCtcpSound()
{
  client->mediaDlg->show();
}

void KEIrc::slotOptPref()
{
  client->optPrefDlg->show();
}

IrcView* KEIrc::ircView()
{
  return view;
}

QDict<QWidget>* KEIrc::windowList()
{
  return &window;
}




