#include <iostream.h>
#include "ircApp.h"
#include "outputHtml.h"


extern HTMLFontManager* pFontManager;

OutputHtml::OutputHtml(QWidget *parent, const char *name, const char *pixdir) :
  KHTMLWidget(parent, name, pixdir)
{

}

void OutputHtml::begin()
{
#ifdef EDEBUG
    cout << "OutputHtml::begin()"<<endl;
#endif
    emit documentStarted();
    
    bIsFrameSet = FALSE;
    bFramesComplete = FALSE;
    framesetStack.clear();
    framesetList.clear();
    frameList.clear();

    
    bgPixmapURL = 0;
    
    stopParser();
    
    baseTarget = "";

    if ( stringTok )
	delete stringTok;
    stringTok = new StringTokenizer;

    if ( ht != 0 )
	delete ht;
    ht = new HTMLTokenizer( this );
    ht->begin();

    writing = true;
}


void OutputHtml::write(const char* str)
{
#ifdef EDEBUG
   cout << "OutputHtml::write("<<str<<")"<<endl;
#endif
   //KHTMLWidget::write(str);
   //return;
   if (str == 0L )
      return;
    
   ht->write(str);

   static const char *end[] = { "</body>", 0 }; 

    if (!painter)
	return;

    const QFont &oldFont = painter->font();

    painter->setFont( *font_stack.top() );

    int lastHeight = docHeight();

    parseCount = granularity;

    
    if ( parseBody( clue, end, TRUE ) )
       stopParser();
       
    clue->calcSize();
    clue->setPos( 0, clue->getAscent() );
    calcAbsolutePos();

    painter->setFont( oldFont );

    painter->setBackgroundColor( backgroundColor() );

    // If the visible rectangle was not filled before the parsing and
    // if we have something to display in the visible area now then repaint.
    
    if ( lastHeight - y_offset < height() * 2 && docHeight() - y_offset > 0 )
	scheduleUpdate( false );
	
    emit documentChanged();
/*
    if (!parsing){
    // Is y_offset too big ?
    if ( docHeight() - y_offset < height() ){
	y_offset = docHeight() - height();
	if ( y_offset < 0 )
	   y_offset = 0;
    }
    // Adjust the scrollbar
    emit scrollVert( y_offset );

    // Is x_offset too big ?
    if ( docWidth() - x_offset < width() ){
      x_offset = docWidth() - width();
      if ( x_offset < 0 )
	 x_offset = 0;
    }	    
    // Adjust the scrollbar
    emit scrollHorz( x_offset );
	
    painter->end();
    delete painter;
    painter = 0;

    // Did we finish the job or are still pictures missing ?
    if ( waitingFileList.count() == 0 && bgPixmapURL.isEmpty() ){
       emit documentDone();
    }

 
    // Now it is time to tell all frames what they should do
    KHTMLView *v;
    KHTMLWidget *w;
    for ( w = frameList.first(); w != 0L; w = frameList.next() ){
       v = w->getView();
       if ( v->getCookie() )
	  v->openURL( v->getCookie() );
       v->show();
    }
 
    HTMLFrameSet *s;
    for ( s = framesetList.first(); s != 0L; s = framesetList.next() ){
       s->show();
    }
    if ( ( s = framesetList.getFirst() ) )
       s->setGeometry( 0, 0, width(), height() );
    bDrawBackground = true;
    }
*/
#ifdef EDEBUG
   cout << "OutputHtml::write()...Ok"<<endl;
#endif
}


void OutputHtml::parse()
{
#ifdef EDEBUG
    cout << "OutputHtml::parse()"<<endl;
#endif
    // if there is no tokenizer then no html has been added
    if ( !ht )
	return;
	
    stopParser();
    
    if ( painter )
    {
	painter->end();
	delete painter;
    }
    painter = new QPainter();
    painter->begin( this );

    tempStrings.clear();

    char *str;
    for ( str = parsedURLs.first(); str; str = parsedURLs.next() )
	delete [] str;
    parsedURLs.clear();
    for ( str = parsedTargets.first(); str; str = parsedTargets.next() )
	delete [] str;
    parsedTargets.clear();

    // Initialize the font stack with the default font.
    italic = false;
    underline = false;
    strikeOut = false;
    weight = QFont::Normal;
    *settings = *defaultSettings;

    colorStack.clear();
    colorStack.push( new QColor( settings->fontBaseColor ) );
    
    font_stack.clear();
    HTMLFont f( settings->fontBaseFace, settings->fontBaseSize );
    f.setTextColor( settings->fontBaseColor );
    const HTMLFont *fp = pFontManager->getFont( f );
    font_stack.push( fp );

    // reset form related stuff
    formList.clear();
    form = 0;
    formSelect = 0;
    inOption = false;
    inTextArea = false;

    inTitle = false;
    bodyParsed = false;

    target = 0;
    url = 0;

    listStack.clear();
    glossaryStack.clear();
    mapList.clear();

    parsing = true;
    indent = 0;
    vspace_inserted = true;
    divAlign = HTMLClue::Left;

    // move to the first token
    ht->first();

    // clear page
    bDrawBackground = true;
    if ( bgPixmap.isNull() )
	painter->eraseRect( 0, 0, width(), height() );
    else
	drawBackground( x_offset, y_offset, 0, 0, width(), height() );

    if ( !bgPixmap.isNull() )
	bgPixmap.resize( -1, -1 );
    else
	bDrawBackground = false;

    if (clue)
	delete clue;
    clue = new HTMLClueV( 0, 0, width() - leftBorder - rightBorder );
    clue->setVAlign( HTMLClue::Top );
    clue->setHAlign( HTMLClue::Left );

    flow = 0;
#ifdef EDEBUG
    cout << "OutputHtml::parse()...Ok"<<endl;
#endif
}



const char* OutputHtml::parseBody( HTMLClueV *_clue, const char *_end[], bool toplevel )
{
#ifdef EDEBUG
    cout << "OutputHtml::parseBody()"<<endl;
#endif
    const char *str;
    
    // Before one can write any objects in the body he usually adds a FlowBox
    // to _clue. Then put all texts, images etc. in the FlowBox.
    // If f == 0L, you have to create a new FlowBox, otherwise you can
    // use the one stored in f. Setting f to 0L means closing the FlowBox.

    while ( ht->hasMoreTokens())
    {
	str = ht->nextToken();

	if ( *str == '\0' )
	    continue;

	if ( *str == ' ' && *(str+1) == '\0' )
	{
	    // if in* is set this text belongs in a form element
	    if ( inOption || inTextArea )
		formText += " ";
	    else if ( inTitle )
		title += " ";
	    else if ( flow != 0)
	    {
		HTMLText *t;
		if ( url || target )
		    t = new HTMLLinkText( " ", currentFont(), painter,
			url, target );
		else
		    t = new HTMLText( " ", currentFont(), painter );
		t->setSeparator( true );
		flow->append( t );
	    }
	}
	else if ( *str != TAG_ESCAPE )
	{
	    // if in* is set this text belongs in a form element
	    if ( inOption || inTextArea )
	    {
		formText += str;
	    }
	    else if ( inTitle )
	    {
		title += str;
	    }
	    else
	    {
		vspace_inserted = false;

		if ( flow == 0 )
		{
		    flow = new HTMLClueFlow( 0, 0, _clue->getMaxWidth() );
		    flow->setIndent( indent );
		    flow->setHAlign( divAlign );
		    _clue->append( flow );
		}
		
    	        if (charsetConverter){
		   QList<KCharsetConversionResult> rl=
		           charsetConverter->multipleConvert(str);
		   KCharsetConversionResult *r;
		   for(r=rl.first();r;r=rl.next()){ 
		      	char *str1=r->copy();
		        HTMLFont f=*currentFont();
			f.setCharset(r->charset());
	                const HTMLFont *fp = pFontManager->getFont( f );
		   	if ( url || target )
		       		flow->append( new HTMLLinkText( str1, fp,
					painter, url, target,TRUE ) );
		   	else
		       		flow->append( new HTMLText( str1, fp,
				        painter,TRUE ) );
		  }		
		}
		else{
		  bool autoDelete;
	          const HTMLFont *fp = currentFont();
		  if (*str=='&'){ // we don't need converter for this
		     char *buffer=new char[strlen(str)+2]; // buffer will never
		                                           // have to be longer
		     int l;
		     const char *str1;
		     KCharsets *charsets=KApplication::getKApplication()->getCharsets();
		     const KCharsetConversionResult &r=charsets->convertTag(str,l);
		     str1=r;					 
		     if (str1 && l){
		       HTMLFont f=*fp;
		       if (r.charset().ok()){
		         f.setCharset(r.charset());
	                 fp = pFontManager->getFont( f );
		       } 
		       strcpy(buffer,str1);
		       strcat(buffer,str+l);
		       str=buffer;
		       autoDelete=TRUE;
		     }  
		  }
		  else autoDelete=FALSE;
		
		  if ( url || target )
		      flow->append( new HTMLLinkText( str, fp, painter,
		  	 url, target,autoDelete ) );
		  else
		      flow->append( new HTMLText( str, fp, painter,autoDelete ) );
		}      
	    }
	}
	else
	{
	    str++;

	    int i = 0;

	    while ( _end[i] != 0 )
	    {
		if ( strncasecmp( str, _end[i], strlen( _end[i] ) ) == 0 )
		{
		    return str;
		}
		i++;
	    }
	    
	    // The tag used for line break when we are in <pre>...</pre>
	    if ( *str == '\n' )
	    {
		// tack a space on the end to ensure the previous line is not
		// zero pixels high
		if ( flow && !flow->hasChildren() )
		    flow->append( new HTMLText( currentFont(), painter ) );
		flow = new HTMLClueH( 0, 0, _clue->getMaxWidth() );
		flow->setIndent( indent );
		_clue->append( flow );
	    }
	    else
		parseOneToken( _clue, str );
	}

	// perhaps we have the frame read complete. So skip the rest
	if ( bFramesComplete )
	{
	    stopParser();
	    return 0;
	}

	if ( toplevel )
	{
	    if ( parseCount <= 0 )
		return 0L;
	}
	parseCount--;
    }

    if ( !ht->hasMoreTokens() && toplevel && !writing )
	stopParser();

#ifdef EDEBUG
    cout << "OutputHtml::parseBody()...Ok"<<endl;
#endif   
    return 0L;
}


void OutputHtml::timerEvent( QTimerEvent * )
{
#ifdef EDEBUG
    cout << "OutputHtml::timerEvent()"<<endl;
#endif
}
