#include <iostream.h>
#include <qregexp.h>
#include <qfile.h>
#include "kprocess.h"
#include "kmsgbox.h"
#include "ircClient.h"
#include "tclCmd.h"
#include "cmd.h"
#include "StringToken.h"
#include "ircApp.h"

TclCmdDict::TclCmdDict()
{
  mDict=QDict<TCL_CMD_TABELLE>(20, FALSE);
  mDict.setAutoDelete(TRUE);
}

TclCmdDict::~TclCmdDict()
{
}

QDict<TCL_CMD_TABELLE>& TclCmdDict::dict()
{
  return mDict;
}

TclCmd::TclCmd(IrcClient* Client, const char*)
{
  client = Client;
  interp = Tcl_CreateInterp();
  Tcl_Init(interp);
  initTclCmd();
  initTclCmdTable();
}

TclCmd::~TclCmd()
{
  Tcl_DeleteInterp(interp);
}

void TclCmd::initTclCmd()
{
#ifdef EDEBUG
  cout << "TclCmd::initTclCmd()"<<endl;
#endif
  Tcl_CreateCommand(interp, "sleep",    tcl_sleep,   (ClientData)client, NULL);

  Tcl_CreateCommand(interp, "nick",  tcl_nick,  (ClientData)client, NULL);
  Tcl_CreateCommand(interp, "server",tcl_server,(ClientData)client, NULL);
  Tcl_CreateCommand(interp, "userinfo",  tcl_userinfo,  (ClientData)client, NULL);
  Tcl_CreateCommand(interp, "userhost",  tcl_userhost,  (ClientData)client, NULL);
  Tcl_CreateCommand(interp, "userserver",tcl_userserver,(ClientData)client, NULL);

  Tcl_CreateCommand(interp, "userlist",  tcl_userlist,  (ClientData)client, NULL);
  Tcl_CreateCommand(interp, "chanlist",  tcl_chanlist,  (ClientData)client, NULL);
  Tcl_CreateCommand(interp, "haveop",    tcl_haveop,    (ClientData)client, NULL);

  Tcl_CreateCommand(interp, "sendraw",    tcl_sendraw,    (ClientData)client, NULL);
  Tcl_CreateCommand(interp, "sendserver", tcl_sendserver, (ClientData)client, NULL);
  Tcl_CreateCommand(interp, "sendchannel",tcl_sendchannel,(ClientData)client, NULL);
  Tcl_CreateCommand(interp, "sendmsg",    tcl_sendmsg,    (ClientData)client, NULL);
  Tcl_CreateCommand(interp, "sendnotice", tcl_sendnotice, (ClientData)client, NULL);
  Tcl_CreateCommand(interp, "sendctcp",   tcl_sendctcp,   (ClientData)client, NULL);
  Tcl_CreateCommand(interp, "sendctcpreply", tcl_sendctcpreply, (ClientData)client, NULL);
  Tcl_CreateCommand(interp, "sendmode",   tcl_sendmode,   (ClientData)client, NULL);
  Tcl_CreateCommand(interp, "sendcmd",    tcl_sendcmd ,   (ClientData)client, NULL);

  Tcl_CreateCommand(interp, "writewnd",   tcl_writewnd,   (ClientData)client, NULL);
  Tcl_CreateCommand(interp, "writeactive",tcl_writeactive,(ClientData)client, NULL);

  Tcl_CreateCommand(interp, "bind",     tcl_bind,     (ClientData)client, NULL);
  Tcl_CreateCommand(interp, "unbind",   tcl_unbind,   (ClientData)client, NULL);
  Tcl_CreateCommand(interp, "setopt",   tcl_setopt,   (ClientData)client, NULL);
  Tcl_CreateCommand(interp, "getopt",   tcl_getopt,   (ClientData)client, NULL);
}

void  TclCmd::initTclCmdTable()
{
  
  cmdTable = new QDict<TclCmdDict>(20, FALSE, TRUE);
  cmdTable->setAutoDelete(TRUE);

  TclCmdDict* tcd;
  cmdTable->insert("MSG",      tcd = new TclCmdDict);
  cmdTable->insert("CHANMSG",  tcd = new TclCmdDict);
  cmdTable->insert("PRIVMSG",  tcd = new TclCmdDict);

  cmdTable->insert("JOIN", tcd = new TclCmdDict);
  cmdTable->insert("PART", tcd = new TclCmdDict);
  cmdTable->insert("TOPIC",tcd = new TclCmdDict);
  cmdTable->insert("KICK", tcd = new TclCmdDict);

  cmdTable->insert("MODE", tcd = new TclCmdDict);
  cmdTable->insert("NICK", tcd = new TclCmdDict);

  cmdTable->insert("CTCP", tcd = new TclCmdDict);
  cmdTable->insert("CTCR", tcd = new TclCmdDict);
  cmdTable->insert("DCC",  tcd = new TclCmdDict);
  cmdTable->insert("TIMER",tcd = new TclCmdDict);
}

int TclCmd::getCmdItem(const char* typ, const char* cmd, QStrList& funcName)
{
#ifdef EDEBUG
  cout << "TclCmd::getCmdItem():"<<typ<<":"<<cmd<<endl;
#endif
  int count=0;
  if (!typ || !cmd || strlen(cmd)<1)
     return count;
  funcName.clear();
  TclCmdDict* cmdDict=getCmdDict(typ);
  if (cmdDict){
     QDictIterator<TCL_CMD_TABELLE> it(cmdDict->dict());
     TCL_CMD_TABELLE* tct;
     for (;tct=it.current();++it){
        int idx, len;
        QRegExp re(it.currentKey(), FALSE, TRUE);
	idx=re.match(cmd, 0, &len);
	if (len==strlen(cmd)){
	   count++;
	   StringToken st(tct->funcName);
	   while(st.hasMoreTokens())
	      funcName.append(st.nextToken());
	}
     }
  } 
#ifdef EDEBUG
  cout << "TclCmd::getCmdItem()...:"<<count<<endl;
#endif
  return count;
}

int TclCmd::evalCmd(QStrList& funcName, const char* argv, QString& Result)
{
#ifdef EDEBUG
  cout << "TclCmd::evalCmd:"<<endl;
#endif
  QString cmd;
  for (cmd=funcName.first();cmd=funcName.current();cmd=funcName.next()){
      QString s = cmd+" "+QString(argv); 
      Tcl_Eval(getInterp(), s.data());
      Result = getInterp()->result;
  }
  return 1;
}

int TclCmd::evalCmd(const char* funcName, const char* argv, QString& Result)
{
#ifdef EDEBUG
  cout << "TclCmd::evalCmd:"<<endl;
#endif
  QString s = QString(funcName)+" "+QString(argv); 
  Tcl_Eval(getInterp(), s.data());
  Result = getInterp()->result;
  return 1;
}

int TclCmd::evalCmd(QStrList& funcName, const char* argv)
{
  QString s;
  return evalCmd(funcName, argv, s);
}

int TclCmd::evalCmd(const char* funcName, const char* argv)
{
  QString s;
  return evalCmd(funcName, argv, s);
}

int TclCmd::evalFile(const char* file)
{
#ifdef EDEBUG
  cout << "TclCmd::evalFile:"<<file<<endl;
#endif
  if (QFile::exists(file)==false){
     KMsgBox::message(0L, "TCL EvalFile Dialog", "File "+QString(file)+" not exist" , 
		      KMsgBox::INFORMATION, "OK");
     return -1;
  }

  int eval=Tcl_EvalFile(getInterp(), (char*)file);
  if (eval!=0){
     KMsgBox::message(0L, "TCL EvalFile Dialog", "Error on loading "+QString(file) , 
		      KMsgBox::INFORMATION, "OK");
     return -1;
  }

  KMsgBox::message(0L, "TCL EvalFile Dialog", "Succeed on loading "+QString(file) , 
		   KMsgBox::INFORMATION, "OK");

  return 1;
}

// add command 
int TclCmd::cmdBind(char* typ, char* flags, char* cmd, char* proc)
{
#ifdef EDEBUG
  cout << "TclCmd::cmdBind():"<<typ<<":"<<flags<<":"<<cmd<<":"<<proc<<endl;
#endif

  TclCmdDict *tcd = cmdTable->find(STR_CONVERT(typ));
  if (!tcd)
     return -1;

 // TIMER flag SECOND funcName
  if (!stricmp(typ, "TIMER")){
     int tmp=atol(cmd);
     if (tmp<1)
        return -1;
     QString qCmd = QString(flags)+cmd;
     tcd->dict().remove(qCmd);
     TCL_CMD_TABELLE* tct = new TCL_CMD_TABELLE;
     tcd->dict().insert(qCmd, tct);
     tct->funcName = proc;
     tct->flag     = tmp;
  }
  else {
     TCL_CMD_TABELLE* tct=tcd->dict().find(cmd);
     if (tct!=0L){
        StringToken st(tct->funcName);
	bool ok=FALSE;
	QRegExp re(STR_CONVERT(proc), FALSE, TRUE);
	while (st.hasMoreTokens() && !ok){
	  int len;
	  QString funcName=st.nextToken();
	  int idx = re.match(funcName, 0, &len);
	  // Noch keinen Eintrag
	  if (len==strlen(proc))
	     ok=TRUE;
	}
	if (!ok)
	  tct->funcName += " "+QString(proc);
     }
     else{
        tct = new TCL_CMD_TABELLE;
        tcd->dict().insert(STR_CONVERT(cmd), tct);
        tct->funcName = proc;
     }
  }

#ifdef EDEBUG
  cout << "TclCmd::cmdBind()...Ok"<<endl;
#endif
  return 1;
}
 

// remove command 
int TclCmd::cmdUnbind(char* typ, char* flags, char* cmd, char* proc)
{
#ifdef EDEBUG
  cout << "TclCmd::cmdUnbind():"<<typ<<":"<<flags<<":"<<cmd<<":"<<proc<<endl;
#endif
  TclCmdDict* tcd = cmdTable->find(STR_CONVERT(typ));
  if (!tcd)
     return -1;

  bool s;
  if (!stricmp(typ, "TIMER")){
     QString qCmd = QString(flags)+cmd;
     s=tcd->dict().remove(qCmd);
  }
  else{
     TCL_CMD_TABELLE* tct=tcd->dict().find(STR_CONVERT(cmd));
     if (tct!=0L){
        StringToken st(tct->funcName);
	tct->funcName = "";
	QRegExp re(STR_CONVERT(proc), FALSE, TRUE);
	while (st.hasMoreTokens()){
	   int len;
	   QString funcName = st.nextToken();
	   int idx = re.match(funcName, 0, &len);
	   if (len!=strlen(funcName))
	      tct->funcName += " "+funcName;
	}
	//if (tct->funcName.isEmpty())
	// bool s=tcd->dict().remove(STR_CONVERT(cmd));
     }
  }

#ifdef EDEBUG
  cout << "TclCmd::cmdUnbind()...Ok"<<endl;
#endif
  return (s?1:-1);
}


// change option
int TclCmd::cmdSetOpt(char* var, char* val, char* defValue)
{
#ifdef EDEBUG
  cout << "TclCmd::cmdSetOpt():"<<var<<":"<<val<<endl;
#endif
  QString entry=var;
  QString value;
  if (defValue)
     value=ircapp->readEntry(entry, QString(defValue));
  else
     value=val;
  ircapp->writeEntry(entry, value);
  Tcl_SetVar(interp, entry.data(), value.data(), TCL_GLOBAL_ONLY);
#ifdef EDEBUG
  cout << "TclCmd::cmdSetOpt()...Ok"<<endl;
#endif
  return 1;
}
 
// get option
int TclCmd::cmdGetOpt(char* var, char* defValue)
{
#ifdef EDEBUG
  cout << "TclCmd::cmdGetOpt():"<<var<<":"<<defValue<<endl;
#endif
  QString entry=var;
  QString value;
  value=ircapp->readEntry(entry, QString(defValue));

  Tcl_SetResult(interp, value.data(), TCL_VOLATILE);
  return 1;
}

void TclCmd::setStartConfig()
{
  QString entry;
  
  entry = "AutoConnect";
  cmdSetOpt(entry.data(), 0L, "No");

  entry = "AutoChangeNick";
  cmdSetOpt(entry.data(), 0L, "No");

  entry = "AutoReconnect";
  cmdSetOpt(entry.data(), 0L, "No");

  entry = "StartActionDelay";
  cmdSetOpt(entry.data(), 0L, "1");
  entry = "ReconnectionDelay";
  cmdSetOpt(entry.data(), 0L, "1");
  entry = "IsonDelay";
  cmdSetOpt(entry.data(), 0L, "1");
}


void TclCmd::setOnPrivMsg()
{
  QString entry;
  
  entry = "OnPrivMsg";
  cmdSetOpt(entry.data(), 0L, "AutoAccept");

  entry = "PrivMsgClientMax";
  cmdSetOpt(entry.data(), 0L, "5");

  // max window
  entry = "PrivMsgDlgMax";
  cmdSetOpt(entry.data(), 0L, "5");

  // timeout
  entry = "PrivMsgDlgTimeout";
  cmdSetOpt(entry.data(), 0L, "30");
}

void TclCmd::setOnInvite()
{
  QString entry;
  
  entry = "OnChannelInvite";
  cmdSetOpt(entry.data(), 0L, "Dialog");

  // max window
  entry = "InvDlgMax";
  cmdSetOpt(entry.data(), 0L, "5");

  // timeout
  entry = "InvDlgTimeout";
  cmdSetOpt(entry.data(), 0L, "30");
}

void TclCmd::setOnKick()
{
  QString entry;
  
  entry = "OnChannelKick";
  cmdSetOpt(entry.data(), 0L, "Quiet");

  // timeout
  entry = "KickDlgTimeout";
  cmdSetOpt(entry.data(), 0L, "30");
}

void TclCmd::setOnDCCChat()
{
  QString entry;
  
  entry = "OnDCCChat";
  cmdSetOpt(entry.data(), 0L, "AutoAccept");

  entry = "DCCChatClientMax";
  cmdSetOpt(entry.data(), 0L, "5");

  // max window
  entry = "DCCChatDlgMax";
  cmdSetOpt(entry.data(), 0L, "5");

  // timeout
  entry = "DCCChatDlgTimeout";
  cmdSetOpt(entry.data(), 0L, "30");
  entry = "DCCChatTimeout";
  cmdSetOpt(entry.data(), 0L, "120");
}

void TclCmd::setOnDCCFile()
{
  QString entry;
  
  entry = "OnDCCFile";
  cmdSetOpt(entry.data(), 0L, "AutoAccept");

  entry = "DCCFileClientMax";
  cmdSetOpt(entry.data(), 0L, "5");

  // max window
  entry = "DCCFileDlgMax";
  cmdSetOpt(entry.data(), 0L, "5");

  // timeout
  entry = "DCCFileDlgTimeout";
  cmdSetOpt(entry.data(), 0L, "30");
  entry = "DCCSendTimeout";
  cmdSetOpt(entry.data(), 0L, "120");

  // On Finished
  entry = "OnFinishedBeep";
  cmdSetOpt(entry.data(), 0L, "No");
  entry = "OnFinishedCloseWindow";
  cmdSetOpt(entry.data(), 0L, "No");

  entry = "DCCSendBlock";
  cmdSetOpt(entry.data(), 0L, "1024");
  entry = "DCCFileDir";
  cmdSetOpt(entry.data(), 0L, ircapp->home.data());
}

void TclCmd::setOnCTCPSound()
{
  QString entry;
  
  entry = "OnCTCPSound";
  cmdSetOpt(entry.data(), 0L, "AutoAccept");

  entry = "OnCTCPSoundOnPlay";
  cmdSetOpt(entry.data(), 0L, "Dontplay");

  entry = "CTCPSoundDir";
  cmdSetOpt(entry.data(), 0L, ircapp->home.data());
}








