#ifndef __TCLCMD_H__
#define __TCLCMD_H__

#define STR_CONVERT(str) ((const char*)str)
#define END_TABELLE { NULL, NULL, -1 }

#include <tcl.h>
#include <qobject.h>
#include <qstring.h>
#include <qstrlist.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef int (*CMDFunc)();
typedef struct {
  QString name;
  CMDFunc func;
  int     access;
} CMD_TABELLE;


// funcName durch Space getrennt
typedef struct {
  int     flag;
  QString funcName;
} TCL_CMD_TABELLE;

#ifdef __cplusplus
}
#endif

class IrcClient;

class TclCmdDict
{
protected:
  QDict<TCL_CMD_TABELLE> mDict;
public:
  TclCmdDict();
  ~TclCmdDict();
  QDict<TCL_CMD_TABELLE>& dict();
};

class TclCmd
{
protected:
  IrcClient*  client;
  Tcl_Interp* interp;
  QDict<TclCmdDict>* cmdTable;
public:
  // Konstructor
  TclCmd(IrcClient* Client, const char* Name=0);
  ~TclCmd();
protected:
  void initTclCmdTable();
  void initTclCmd();
public:
  int  cmdBind  (char* typ, char* flags, char* cmd, char* proc);
  int  cmdUnbind(char* typ, char* flags, char* cmd, char* proc);
  int  cmdSetOpt(char* var, char* val,   char* defVal=0L);
  int  cmdGetOpt(char* var, char* defVal="");
  Tcl_Interp* getInterp(){return interp;}
  QDict<TclCmdDict>* getCmdTable(){
    return cmdTable;
  }
  inline TclCmdDict* getCmdDict(const char* s){
    return getCmdTable()->find(s);
  }
  int getCmdItem(const char* typ, const char* cmd, QStrList& funcName);
  int evalCmd   (QStrList&   funcName, const char* argv);
  int evalCmd   (const char* funcName, const char* argv);
  int evalCmd   (QStrList&   funcName, const char* argv, QString& Result);
  int evalCmd   (const char* funcName, const char* argv, QString& Result);
  int evalFile  (const char* file);

  // variable fuer Einstellung initialisieren
  void setStartConfig();
  void setOnPrivMsg();
  void setOnInvite();
  void setOnKick();
  void setOnDCCChat();
  void setOnDCCFile();
  void setOnCTCPSound();
};

#endif
